from datetime import timedelta

from django.conf import settings
from django.utils import timezone
from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from arkindex.documents.models import CorpusExport, CorpusExportState
from arkindex.project.serializer_fields import EnumField
from arkindex.users.serializers import SimpleUserSerializer


class CorpusExportSerializer(serializers.ModelSerializer):
    user = SimpleUserSerializer(read_only=True)
    state = EnumField(CorpusExportState, read_only=True)

    class Meta:
        model = CorpusExport
        fields = ("id", "created", "updated", "corpus_id", "user", "state", "source",)

    def validate(self, data):
        corpus = self.context["corpus"]
        source = data.get("source", "default")
        # Check that there is no export already running for this corpus
        if corpus.exports.filter(state__in=(CorpusExportState.Created, CorpusExportState.Running)).exists():
            raise ValidationError("An export is already running for this corpus.")
        # Check that there is no available completed export from the same source created less than {EXPORT_TTL_SECONDS}
        # ago for this corpus
        available_exports = corpus.exports.filter(
            state=CorpusExportState.Done,
            source=source,
            created__gte=timezone.now() - timedelta(seconds=settings.EXPORT_TTL_SECONDS)
        )
        if available_exports.exists():
            raise ValidationError(f"An export has already been made for this corpus in the last {settings.EXPORT_TTL_SECONDS} seconds.")

        data["corpus"] = corpus
        data["source"] = source
        return data

    def create(self, validated_data):
        export = CorpusExport.objects.create(
            user=self.context["request"].user,
            corpus=validated_data["corpus"],
            source=validated_data["source"]
        )
        export.start()
        return export
